@@ -9,6 +9,7 @@ import java.awt.Point;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.MouseEvent;
+import java.net.URL;
 import java.util.Collections;
 import java.util.LinkedList;
 import java.util.List;
@@ -43,14 +44,20 @@ import org.openstreetmap.gui.jmapviewer.tilesources.OsmTileSource;
  */
 public class JMapViewer extends JPanel implements TileLoaderListener {
 
+    /** whether debug mode is enabled or not */
     public static boolean debug;
 
+    /** option to reverse zoom direction with mouse wheel */
+    public static boolean zoomReverseWheel;
+
     /**
      * Vectors for clock-wise tile painting
      */
     private static final Point[] move = {new Point(1, 0), new Point(0, 1), new Point(-1, 0), new Point(0, -1)};
 
+    /** Maximum zoom level */
     public static final int MAX_ZOOM = 22;
+    /** Minimum zoom level */
     public static final int MIN_ZOOM = 0;
 
     protected transient List<MapMarker> mapMarkerList;
@@ -81,8 +88,13 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
     protected JButton zoomInButton;
     protected JButton zoomOutButton;
 
+    /**
+     * Apparence of zoom controls.
+     */
     public enum ZOOM_BUTTON_STYLE {
+        /** Zoom buttons are displayed horizontally (default) */
         HORIZONTAL,
+        /** Zoom buttons are displayed vertically */
         VERTICAL
     }
 
@@ -109,7 +121,8 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
     /**
      * Creates a new {@link JMapViewer} instance.
      * @param tileCache The cache where to store tiles
-     * @deprecated @param downloadThreadCount not used anymore
+     * @param downloadThreadCount not used anymore
+     * @deprecated use {@link #JMapViewer(TileCache)}
      */
     @Deprecated
     public JMapViewer(TileCache tileCache, int downloadThreadCount) {
@@ -157,10 +170,11 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         zoomSlider.setFocusable(false);
         add(zoomSlider);
         int size = 18;
-        try {
-            ImageIcon icon = new ImageIcon(JMapViewer.class.getResource("images/plus.png"));
+        URL url = JMapViewer.class.getResource("images/plus.png");
+        if (url != null) {
+            ImageIcon icon = new ImageIcon(url);
             zoomInButton = new JButton(icon);
-        } catch (Exception e) {
+        } else {
             zoomInButton = new JButton("+");
             zoomInButton.setFont(new Font("sansserif", Font.BOLD, 9));
             zoomInButton.setMargin(new Insets(0, 0, 0, 0));
@@ -175,10 +189,11 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         });
         zoomInButton.setFocusable(false);
         add(zoomInButton);
-        try {
-            ImageIcon icon = new ImageIcon(JMapViewer.class.getResource("images/minus.png"));
+        url = JMapViewer.class.getResource("images/minus.png");
+        if (url != null) {
+            ImageIcon icon = new ImageIcon(url);
             zoomOutButton = new JButton(icon);
-        } catch (Exception e) {
+        } else {
             zoomOutButton = new JButton("-");
             zoomOutButton.setFont(new Font("sansserif", Font.BOLD, 9));
             zoomOutButton.setMargin(new Insets(0, 0, 0, 0));
@@ -227,10 +242,23 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         setDisplayPosition(mapPoint, p.x, p.y, zoom);
     }
 
+    /**
+     * Sets the display position.
+     * @param x X coordinate
+     * @param y Y coordinate
+     * @param zoom zoom level, between {@link #MIN_ZOOM} and {@link #MAX_ZOOM}
+     */
     public void setDisplayPosition(int x, int y, int zoom) {
         setDisplayPosition(new Point(getWidth() / 2, getHeight() / 2), x, y, zoom);
     }
 
+    /**
+     * Sets the display position.
+     * @param mapPoint map point
+     * @param x X coordinate
+     * @param y Y coordinate
+     * @param zoom zoom level, between {@link #MIN_ZOOM} and {@link #MAX_ZOOM}
+     */
     public void setDisplayPosition(Point mapPoint, int x, int y, int zoom) {
         if (zoom > tileController.getTileSource().getMaxZoom() || zoom < MIN_ZOOM)
             return;
@@ -280,7 +308,7 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         int mapZoomMax = tileController.getTileSource().getMaxZoom();
 
         if (markers && mapMarkerList != null) {
-            synchronized (mapMarkerList) {
+            synchronized (this) {
                 for (MapMarker marker : mapMarkerList) {
                     if (marker.isVisible()) {
                         Point p = tileSource.latLonToXY(marker.getCoordinate(), mapZoomMax);
@@ -294,7 +322,7 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         }
 
         if (rectangles && mapRectangleList != null) {
-            synchronized (mapRectangleList) {
+            synchronized (this) {
                 for (MapRectangle rectangle : mapRectangleList) {
                     if (rectangle.isVisible()) {
                         Point bottomRight = tileSource.latLonToXY(rectangle.getBottomRight(), mapZoomMax);
@@ -309,7 +337,7 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         }
 
         if (polygons && mapPolygonList != null) {
-            synchronized (mapPolygonList) {
+            synchronized (this) {
                 for (MapPolygon polygon : mapPolygonList) {
                     if (polygon.isVisible()) {
                         for (ICoordinate c : polygon.getPoints()) {
@@ -454,8 +482,8 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
      * @return Integer the radius in pixels
      */
     public Integer getLatOffset(double lat, double lon, double offset, boolean checkOutside) {
-        Point p = tileSource.latLonToXY(lat, lon, zoom);
-        int y = p.y - center.y - getHeight() / 2;
+        Point p = tileSource.latLonToXY(lat + offset, lon, zoom);
+        int y = p.y - (center.y - getHeight() / 2);
         if (checkOutside && (y < 0 || y > getHeight())) {
             return null;
         }
@@ -628,7 +656,7 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         }
 
         if (mapPolygonsVisible && mapPolygonList != null) {
-            synchronized (mapPolygonList) {
+            synchronized (this) {
                 for (MapPolygon polygon : mapPolygonList) {
                     if (polygon.isVisible())
                         paintPolygon(g, polygon);
@@ -637,7 +665,7 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         }
 
         if (mapRectanglesVisible && mapRectangleList != null) {
-            synchronized (mapRectangleList) {
+            synchronized (this) {
                 for (MapRectangle rectangle : mapRectangleList) {
                     if (rectangle.isVisible())
                         paintRectangle(g, rectangle);
@@ -646,7 +674,7 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         }
 
         if (mapMarkersVisible && mapMarkerList != null) {
-            synchronized (mapMarkerList) {
+            synchronized (this) {
                 for (MapMarker marker : mapMarkerList) {
                     if (marker.isVisible())
                         paintMarker(g, marker);
@@ -863,8 +891,7 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
      * derived implementations for adapting zoom dependent values. The new zoom
      * level can be obtained via {@link #getZoom()}.
      *
-     * @param oldZoom
-     *            the previous zoom level
+     * @param oldZoom the previous zoom level
      */
     protected void zoomChanged(int oldZoom) {
         zoomSlider.setToolTipText("Zoom level " + zoom);
@@ -874,15 +901,27 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         zoomInButton.setEnabled(zoom < tileController.getTileSource().getMaxZoom());
     }
 
+    /**
+     * Determines whether the tile grid is visible or not.
+     * @return {@code true} if the tile grid is visible, {@code false} otherwise
+     */
     public boolean isTileGridVisible() {
         return tileGridVisible;
     }
 
+    /**
+     * Sets whether the tile grid is visible or not.
+     * @param tileGridVisible {@code true} if the tile grid is visible, {@code false} otherwise
+     */
     public void setTileGridVisible(boolean tileGridVisible) {
         this.tileGridVisible = tileGridVisible;
         repaint();
     }
 
+    /**
+     * Determines whether {@link MapMarker}s are painted or not.
+     * @return {@code true} if {@link MapMarker}s are painted, {@code false} otherwise
+     */
     public boolean getMapMarkersVisible() {
         return mapMarkersVisible;
     }
@@ -899,88 +938,157 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         repaint();
     }
 
+    /**
+     * Sets the list of {@link MapMarker}s.
+     * @param mapMarkerList list of {@link MapMarker}s
+     */
     public void setMapMarkerList(List<MapMarker> mapMarkerList) {
         this.mapMarkerList = mapMarkerList;
         repaint();
     }
 
+    /**
+     * Returns the list of {@link MapMarker}s.
+     * @return list of {@link MapMarker}s
+     */
     public List<MapMarker> getMapMarkerList() {
         return mapMarkerList;
     }
 
+    /**
+     * Sets the list of {@link MapRectangle}s.
+     * @param mapRectangleList list of {@link MapRectangle}s
+     */
     public void setMapRectangleList(List<MapRectangle> mapRectangleList) {
         this.mapRectangleList = mapRectangleList;
         repaint();
     }
 
+    /**
+     * Returns the list of {@link MapRectangle}s.
+     * @return list of {@link MapRectangle}s
+     */
     public List<MapRectangle> getMapRectangleList() {
         return mapRectangleList;
     }
 
+    /**
+     * Sets the list of {@link MapPolygon}s.
+     * @param mapPolygonList list of {@link MapPolygon}s
+     */
     public void setMapPolygonList(List<MapPolygon> mapPolygonList) {
         this.mapPolygonList = mapPolygonList;
         repaint();
     }
 
+    /**
+     * Returns the list of {@link MapPolygon}s.
+     * @return list of {@link MapPolygon}s
+     */
     public List<MapPolygon> getMapPolygonList() {
         return mapPolygonList;
     }
 
+    /**
+     * Add a {@link MapMarker}.
+     * @param marker map marker to add
+     */
     public void addMapMarker(MapMarker marker) {
         mapMarkerList.add(marker);
         repaint();
     }
 
+    /**
+     * Remove a {@link MapMarker}.
+     * @param marker map marker to remove
+     */
     public void removeMapMarker(MapMarker marker) {
         mapMarkerList.remove(marker);
         repaint();
     }
 
+    /**
+     * Remove all {@link MapMarker}s.
+     */
     public void removeAllMapMarkers() {
         mapMarkerList.clear();
         repaint();
     }
 
+    /**
+     * Add a {@link MapRectangle}.
+     * @param rectangle map rectangle to add
+     */
     public void addMapRectangle(MapRectangle rectangle) {
         mapRectangleList.add(rectangle);
         repaint();
     }
 
+    /**
+     * Remove a {@link MapRectangle}.
+     * @param rectangle map rectangle to remove
+     */
     public void removeMapRectangle(MapRectangle rectangle) {
         mapRectangleList.remove(rectangle);
         repaint();
     }
 
+    /**
+     * Remove all {@link MapRectangle}s.
+     */
     public void removeAllMapRectangles() {
         mapRectangleList.clear();
         repaint();
     }
 
+    /**
+     * Add a {@link MapPolygon}.
+     * @param polygon map polygon to add
+     */
     public void addMapPolygon(MapPolygon polygon) {
         mapPolygonList.add(polygon);
         repaint();
     }
 
+    /**
+     * Remove a {@link MapPolygon}.
+     * @param polygon map polygon to remove
+     */
     public void removeMapPolygon(MapPolygon polygon) {
         mapPolygonList.remove(polygon);
         repaint();
     }
 
+    /**
+     * Remove all {@link MapPolygon}s.
+     */
     public void removeAllMapPolygons() {
         mapPolygonList.clear();
         repaint();
     }
 
+    /**
+     * Sets whether zoom controls are displayed or not.
+     * @param visible {@code true} if zoom controls are displayed, {@code false} otherwise
+     */
     public void setZoomContolsVisible(boolean visible) {
         zoomSlider.setVisible(visible);
         zoomInButton.setVisible(visible);
         zoomOutButton.setVisible(visible);
     }
 
+    /**
+     * Determines whether zoom controls are displayed or not.
+     * @return {@code true} if zoom controls are displayed, {@code false} otherwise
+     */
     public boolean getZoomControlsVisible() {
         return zoomSlider.isVisible();
     }
 
+    /**
+     * Sets the tile source.
+     * @param tileSource tile source
+     */
     public void setTileSource(TileSource tileSource) {
         if (tileSource.getMaxZoom() > MAX_ZOOM)
             throw new RuntimeException("Maximum zoom level too high");
@@ -1006,12 +1114,16 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         repaint();
     }
 
+    /**
+     * Determines whether the {@link MapRectangle}s are painted or not.
+     * @return {@code true} if the {@link MapRectangle}s are painted, {@code false} otherwise
+     */
     public boolean isMapRectanglesVisible() {
         return mapRectanglesVisible;
     }
 
     /**
-     * Enables or disables painting of the {@link MapRectangle}
+     * Enables or disables painting of the {@link MapRectangle}s.
      *
      * @param mapRectanglesVisible {@code true} to enable painting of rectangles
      * @see #addMapRectangle(MapRectangle)
@@ -1022,12 +1134,16 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         repaint();
     }
 
+    /**
+     * Determines whether the {@link MapPolygon}s are painted or not.
+     * @return {@code true} if the {@link MapPolygon}s are painted, {@code false} otherwise
+     */
     public boolean isMapPolygonsVisible() {
         return mapPolygonsVisible;
     }
 
     /**
-     * Enables or disables painting of the {@link MapPolygon}
+     * Enables or disables painting of the {@link MapPolygon}s.
      *
      * @param mapPolygonsVisible {@code true} to enable painting of polygons
      * @see #addMapPolygon(MapPolygon)
@@ -1038,35 +1154,47 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         repaint();
     }
 
+    /**
+     * Determines whether scroll wrap is enabled or not.
+     * @return {@code true} if scroll wrap is enabled, {@code false} otherwise
+     */
     public boolean isScrollWrapEnabled() {
         return scrollWrapEnabled;
     }
 
+    /**
+     * Sets whether scroll wrap is enabled or not.
+     * @param scrollWrapEnabled {@code true} if scroll wrap is enabled, {@code false} otherwise
+     */
     public void setScrollWrapEnabled(boolean scrollWrapEnabled) {
         this.scrollWrapEnabled = scrollWrapEnabled;
         repaint();
     }
 
+    /**
+     * Returns the zoom controls apparence style (horizontal/vertical).
+     * @return {@link ZOOM_BUTTON_STYLE#VERTICAL} or {@link ZOOM_BUTTON_STYLE#HORIZONTAL}
+     */
     public ZOOM_BUTTON_STYLE getZoomButtonStyle() {
         return zoomButtonStyle;
     }
 
+    /**
+     * Sets the zoom controls apparence style (horizontal/vertical).
+     * @param style {@link ZOOM_BUTTON_STYLE#VERTICAL} or {@link ZOOM_BUTTON_STYLE#HORIZONTAL}
+     */
     public void setZoomButtonStyle(ZOOM_BUTTON_STYLE style) {
         zoomButtonStyle = style;
         if (zoomSlider == null || zoomInButton == null || zoomOutButton == null) {
             return;
         }
         switch (style) {
-        case HORIZONTAL:
-            zoomSlider.setBounds(10, 10, 30, 150);
-            zoomInButton.setBounds(4, 155, 18, 18);
-            zoomOutButton.setBounds(26, 155, 18, 18);
-            break;
         case VERTICAL:
             zoomSlider.setBounds(10, 27, 30, 150);
             zoomInButton.setBounds(14, 8, 20, 20);
             zoomOutButton.setBounds(14, 176, 20, 20);
             break;
+        case HORIZONTAL:
         default:
             zoomSlider.setBounds(10, 10, 30, 150);
             zoomInButton.setBounds(4, 155, 18, 18);
@@ -1076,6 +1204,10 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         repaint();
     }
 
+    /**
+     * Returns the tile controller.
+     * @return the tile controller
+     */
     public TileController getTileController() {
         return tileController;
     }
@@ -1089,10 +1221,18 @@ public class JMapViewer extends JPanel implements TileLoaderListener {
         return tileController.getTileCache();
     }
 
+    /**
+     * Sets the tile loader.
+     * @param loader tile loader
+     */
     public void setTileLoader(TileLoader loader) {
         tileController.setTileLoader(loader);
     }
 
+    /**
+     * Returns attribution.
+     * @return attribution
+     */
     public AttributionSupport getAttribution() {
         return attribution;
     }
