@@ -81,7 +81,7 @@ public class Tile {
     private static BufferedImage loadImage(String path) {
         try {
             return ImageIO.read(JMapViewer.class.getResourceAsStream(path));
-        } catch (Exception ex) {
+        } catch (IOException | IllegalArgumentException ex) {
             ex.printStackTrace();
             return null;
         }
@@ -227,6 +227,13 @@ public class Tile {
         return zoom;
     }
 
+    /**
+     * @return tile indexes as TileXY object
+     */
+    public TileXY getTileXY() {
+        return new TileXY(xtile, ytile);
+    }
+
     public BufferedImage getImage() {
         return image;
     }
@@ -361,6 +368,10 @@ public class Tile {
         return error_message;
     }
 
+    public void setError(Exception e) {
+        setError(e.toString());
+    }
+
     public void setError(String message) {
         error = true;
         setImage(ERROR_IMAGE);
