@@ -1,7 +1,6 @@
+// License: GPL. For details, see Readme.txt file.
 package org.openstreetmap.gui.jmapviewer.interfaces;
 
-//License: GPL. Copyright 2009 by Stefan Zeller
-
 import java.awt.Graphics;
 import java.awt.Point;
 
@@ -15,17 +14,17 @@ import org.openstreetmap.gui.jmapviewer.JMapViewer;
  * @see JMapViewer#addMapRectangle(MapRectangle)
  * @see JMapViewer#getMapRectangleList()
  */
-public interface MapRectangle extends MapObject{
+public interface MapRectangle extends MapObject {
 
     /**
      * @return Latitude/Longitude of top left of rectangle
      */
-    public Coordinate getTopLeft();
+    Coordinate getTopLeft();
 
     /**
      * @return Latitude/Longitude of bottom right of rectangle
      */
-    public Coordinate getBottomRight();
+    Coordinate getBottomRight();
 
     /**
      * Paints the map rectangle on the map. The <code>topLeft</code> and
@@ -35,5 +34,5 @@ public interface MapRectangle extends MapObject{
      * @param topLeft lop left edge of painting region
      * @param bottomRight bottom right edge of painting region
      */
-    public void paint(Graphics g, Point topLeft, Point bottomRight);
+    void paint(Graphics g, Point topLeft, Point bottomRight);
 }
