@@ -3,12 +3,14 @@ package org.openstreetmap.gui.jmapviewer.tilesources;
 
 import java.awt.Point;
 import java.io.IOException;
+import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
+import java.util.Set;
 
-import org.openstreetmap.gui.jmapviewer.Coordinate;
 import org.openstreetmap.gui.jmapviewer.OsmMercator;
 import org.openstreetmap.gui.jmapviewer.Tile;
 import org.openstreetmap.gui.jmapviewer.TileXY;
@@ -25,10 +27,10 @@ public abstract class AbstractTMSTileSource extends AbstractTileSource {
     protected String name;
     protected String baseUrl;
     protected String id;
-    private final Map<String, String> noTileHeaders;
+    private final Map<String, Set<String>> noTileHeaders;
+    private final Map<String, Set<String>> noTileChecksums;
     private final Map<String, String> metadataHeaders;
     protected int tileSize;
-    protected OsmMercator osmMercator;
 
     /**
      * Creates an instance based on TileSource information
@@ -43,9 +45,9 @@ public abstract class AbstractTMSTileSource extends AbstractTileSource {
         }
         this.id = info.getUrl();
         this.noTileHeaders = info.getNoTileHeaders();
+        this.noTileChecksums = info.getNoTileChecksums();
         this.metadataHeaders = info.getMetadataHeaders();
         this.tileSize = info.getTileSize();
-        this.osmMercator = new OsmMercator(this.tileSize);
     }
 
     /**
@@ -122,19 +124,6 @@ public abstract class AbstractTMSTileSource extends AbstractTileSource {
         return tileSize;
     }
 
-    @Override
-    public double getDistance(double lat1, double lon1, double lat2, double lon2) {
-        return osmMercator.getDistance(lat1, lon1, lat2, lon2);
-    }
-
-    @Override
-    public Point latLonToXY(double lat, double lon, int zoom) {
-        return new Point(
-                (int) osmMercator.lonToX(lon, zoom),
-                (int) osmMercator.latToY(lat, zoom)
-                );
-    }
-
     @Override
     public Point latLonToXY(ICoordinate point, int zoom) {
         return latLonToXY(point.getLat(), point.getLon(), zoom);
@@ -145,22 +134,6 @@ public abstract class AbstractTMSTileSource extends AbstractTileSource {
         return xyToLatLon(point.x, point.y, zoom);
     }
 
-    @Override
-    public ICoordinate xyToLatLon(int x, int y, int zoom) {
-        return new Coordinate(
-                osmMercator.yToLat(y, zoom),
-                osmMercator.xToLon(x, zoom)
-                );
-    }
-
-    @Override
-    public TileXY latLonToTileXY(double lat, double lon, int zoom) {
-        return new TileXY(
-                osmMercator.lonToX(lon, zoom) / tileSize,
-                osmMercator.latToY(lat, zoom) / tileSize
-                );
-    }
-
     @Override
     public TileXY latLonToTileXY(ICoordinate point, int zoom) {
         return latLonToTileXY(point.getLat(), point.getLon(), zoom);
@@ -176,14 +149,6 @@ public abstract class AbstractTMSTileSource extends AbstractTileSource {
         return tileXYToLatLon(tile.getXtile(), tile.getYtile(), tile.getZoom());
     }
 
-    @Override
-    public ICoordinate tileXYToLatLon(int x, int y, int zoom) {
-        return new Coordinate(
-                osmMercator.yToLat(y * tileSize, zoom),
-                osmMercator.xToLon(x * tileSize, zoom)
-                );
-    }
-
     @Override
     public int getTileXMax(int zoom) {
         return getTileMax(zoom);
@@ -207,17 +172,45 @@ public abstract class AbstractTMSTileSource extends AbstractTileSource {
     @Override
     public boolean isNoTileAtZoom(Map<String, List<String>> headers, int statusCode, byte[] content) {
         if (noTileHeaders != null && headers != null) {
-            for (Entry<String, String> searchEntry: noTileHeaders.entrySet()) {
+            for (Entry<String, Set<String>> searchEntry: noTileHeaders.entrySet()) {
                 List<String> headerVals = headers.get(searchEntry.getKey());
                 if (headerVals != null) {
                     for (String headerValue: headerVals) {
-                        if (headerValue.matches(searchEntry.getValue())) {
-                            return true;
+                        for (String val: searchEntry.getValue()) {
+                            if (headerValue.matches(val)) {
+                                return true;
+                            }
                         }
                     }
                 }
             }
         }
+        if (noTileChecksums != null && content != null) {
+            for (Entry<String, Set<String>> searchEntry: noTileChecksums.entrySet()) {
+                MessageDigest md = null;
+                try {
+                    md = MessageDigest.getInstance(searchEntry.getKey());
+                } catch (NoSuchAlgorithmException e) {
+                    break;
+                }
+                byte[] byteDigest = md.digest(content);
+                final int len = byteDigest.length;
+
+                char[] hexChars = new char[len * 2];
+                for (int i = 0, j = 0; i < len; i++) {
+                    final int v = byteDigest[i];
+                    int vn = (v & 0xf0) >> 4;
+                    hexChars[j++] = (char) (vn + (vn >= 10 ? 'a'-10 : '0'));
+                    vn = (v & 0xf);
+                    hexChars[j++] = (char) (vn + (vn >= 10 ? 'a'-10 : '0'));
+                }
+                for (String val: searchEntry.getValue()) {
+                    if (new String(hexChars).equalsIgnoreCase(val)) {
+                        return true;
+                    }
+                }
+            }
+        }
         return super.isNoTileAtZoom(headers, statusCode, content);
     }
 
