@@ -1,9 +1,13 @@
 // License: GPL. For details, see Readme.txt file.
 package org.openstreetmap.gui.jmapviewer.tilesources;
 
+import java.awt.Point;
 import java.util.Random;
 
+import org.openstreetmap.gui.jmapviewer.Coordinate;
 import org.openstreetmap.gui.jmapviewer.OsmMercator;
+import org.openstreetmap.gui.jmapviewer.TileXY;
+import org.openstreetmap.gui.jmapviewer.interfaces.ICoordinate;
 
 /**
  * This tilesource uses different to OsmMercator projection.
@@ -21,15 +25,6 @@ public class ScanexTileSource extends TMSTileSource {
     private static final int DEFAULT_MAXZOOM = 14;
     private static final String API_KEY = "4018C5A9AECAD8868ED5DEB2E41D09F7";
 
-    // Latitude to Y and back calculations.
-
-    /** radius of Earth at equator, m */
-    private static double RADIUS_E = 6378137;
-    /** equator length, m */
-    private static double EQUATOR = 40075016.68557849;
-    /** eccentricity of Earth's ellipsoid */
-    private static double E = 0.0818191908426;
-
     private enum ScanexLayer {
         IRS("irs", "/TileSender.ashx?ModeKey=tile&MapName=F7B8CF651682420FA1749D894C8AD0F6&LayerName=BAC78D764F0443BD9AF93E7A998C9F5B"),
         SPOT("spot", "/TileSender.ashx?ModeKey=tile&MapName=F7B8CF651682420FA1749D894C8AD0F6&LayerName=F51CE95441284AF6B2FC319B609C7DEC");
@@ -53,6 +48,7 @@ public class ScanexTileSource extends TMSTileSource {
 
     /** IRS by default */
     private ScanexLayer layer = ScanexLayer.IRS;
+    private TemplatedTMSTileSource TemplateSource = null;
 
     /** cached latitude used in {@link #tileYToLat(double, int)} */
     private double cachedLat;
@@ -65,6 +61,15 @@ public class ScanexTileSource extends TMSTileSource {
         super(info);
         String url = info.getUrl();
 
+        /**
+         * The formulae in tileYToLat() and latToTileY() have 2^8
+         * hardcoded in them, so explicitly state that.  For now
+         * the assignment matches OsmMercator.DEFAUL_TILE_SIZE, and
+         * thus is extraneous.  But let it be there just in case if
+         * OsmMercator changes.
+         */
+        this.tileSize = 256;
+
         for (ScanexLayer slayer : ScanexLayer.values()) {
             if (url.equalsIgnoreCase(slayer.getName())) {
                 this.layer = slayer;
@@ -72,9 +77,12 @@ public class ScanexTileSource extends TMSTileSource {
                 this.baseUrl = DEFAULT_URL;
                 if (maxZoom == 0)
                     this.maxZoom = DEFAULT_MAXZOOM;
-                break;
+                return;
             }
         }
+        /** If not "irs" or "spot" keyword, then a custom URL. */
+        TemplatedTMSTileSource.checkUrl(info.getUrl());
+        this.TemplateSource = new TemplatedTMSTileSource(info);
     }
 
     @Override
@@ -82,6 +90,14 @@ public class ScanexTileSource extends TMSTileSource {
         return "jpeg";
     }
 
+   @Override
+    public String getTileUrl(int zoom, int tilex, int tiley) {
+        if (this.TemplateSource != null)
+            return this.TemplateSource.getTileUrl(zoom, tilex, tiley);
+        else
+            return this.getBaseUrl() + getTilePath(zoom, tilex, tiley);
+    }
+
     @Override
     public String getTilePath(int zoom, int tilex, int tiley) {
         int tmp = (int) Math.pow(2.0, zoom - 1);
@@ -92,6 +108,50 @@ public class ScanexTileSource extends TMSTileSource {
         return this.layer.getUri() + "&apikey=" + API_KEY + "&x=" + tilex + "&y=" + tiley + "&z=" + zoom;
     }
 
+    // Latitude to Y and back calculations.
+    private static double RADIUS_E = 6378137;   /* radius of Earth at equator, m */
+    private static double EQUATOR = 40075016.68557849; /* equator length, m */
+    private static double E = 0.0818191908426;  /* eccentricity of Earth's ellipsoid */
+
+    @Override
+    public Point latLonToXY(double lat, double lon, int zoom) {
+        return new Point(
+                (int) osmMercator.lonToX(lon, zoom),
+                (int) latToTileY(lat, zoom)
+                );
+    }
+
+    @Override
+    public ICoordinate xyToLatLon(int x, int y, int zoom) {
+        return new Coordinate(
+                tileYToLat((double) y, zoom),
+                osmMercator.xToLon(x, zoom)
+                );
+    }
+
+    @Override
+    public TileXY latLonToTileXY(double lat, double lon, int zoom) {
+        return new TileXY(
+                osmMercator.lonToX(lon, zoom) / getTileSize(),
+                latToTileY(lat, zoom)
+                );
+    }
+
+    @Override
+    public ICoordinate tileXYToLatLon(int x, int y, int zoom) {
+        return new Coordinate(
+                tileYToLat((double) y, zoom),
+                osmMercator.xToLon(x * getTileSize(), zoom)
+                );
+    }
+
+    private double latToTileY(double lat, int zoom) {
+        double tmp = Math.tan(Math.PI/4 * (1 + lat/90));
+        double pow = Math.pow(Math.tan(Math.PI/4 + Math.asin(E * Math.sin(Math.toRadians(lat)))/2), E);
+
+        return (EQUATOR/2 - (RADIUS_E * Math.log(tmp/pow))) * Math.pow(2.0, zoom) / EQUATOR;
+    }
+
     /*
      * To solve inverse formula latitude = f(y) we use
      * Newton's method. We cache previous calculated latitude,
